import chimera, numpy as np, Tkinter as tk
from chimera.baseDialog import ModelessDialog
from time import sleep

#-------------------------------------------------------------------------
# Before using this strip chart, the following gloabal variables and
# functions have to be defined:

# Dimensions of canvas and limits on y-values:
CANVAS_H = 200      # Canvas plot height
CANVAS_W = 800      # Canvas plot width
Y_MIN = -2.1        # Minimum expected y value.
Y_MAX = 2.2         # Maximum expected y value.
# Lists for horizontal grid lines
GRID_DARK_L = [0.0] 
GRID_GRAY_L = [2.0, 1.5, 1.0, 0.5, -0.5, -1.0, -1.5, -2.0]

PLOT_TITLE = " sin(0.02x) + cos(0.07x)"

EVALUATIONS_MAX = 1000  # Maximum number of evaluations of F(x).

def F(x):
    return np.sin(x*.02) + np.cos(x*0.07)  # Same as plot title?

#-------------------------------------------------------------------------
class scrollingPlot(ModelessDialog):
    buttons = ("Close")
    help = False    # Leave this out to get a Help button.
    title = "Strip Chart"

    def fillInUI(self, parent):
        # Text to be displayed at top of the dialog window:
        topText = tk.Label(parent, text =  PLOT_TITLE, 
                           font = ("Helvetica", 11, "normal"))
        topText.grid(row = 0, column = 0, columnspan = 2, pady = 4, sticky = tk.W)

        #-------------------------------------------------------------------------
        # Setting up two buttons:
        buttonsFrame = tk.Frame(parent, bd = 2, relief = tk.RIDGE)
        buttonsFrame.grid(row = 1, column = 0, padx = 4, pady = 4, sticky = tk.W)
                
        self.startBtn = tk.Button(buttonsFrame, text = " Start ", command = self.startBtnFunc)
        self.startBtn.grid(row = 0, column = 0, padx = 4, pady = 4)

        self.stopBtn = tk.Button(buttonsFrame, text = " Stop ", command = self.stopBtnFunc)
        self.stopBtn.grid(row = 0, column = 1, padx = 4, pady = 4)
        
        #-------------------------------------------------------------------------
        # Set up a canvas:
        canvasFrame = tk.Frame(parent, bd = 2, relief = tk.RIDGE)
        canvasFrame.grid(row = 3, column = 0, padx = 4, pady = 4, sticky = tk.E)
        canvasFrame.grid_rowconfigure(0, weight = 1)
        canvasFrame.grid_columnconfigure(0, weight = 1)

        yAxis = tk.Canvas(canvasFrame, width = 60,
                          height = CANVAS_H, bg = "white")
        yAxis.grid(row = 0, column = 0, pady = 4)

        self.stripChart = tk.Canvas(canvasFrame, width = CANVAS_W,
                                   height = CANVAS_H, bg = "white")
        self.stripChart.grid(row = 0, column = 1, pady = 4)

        hbar = tk.Scrollbar(canvasFrame, orient = tk.HORIZONTAL,
                            command = self.stripChart.xview)
        hbar.grid(row = 1, column = 1, sticky = tk.E + tk.W)
        self.stripChart.configure(xscrollcommand = hbar.set)
        #self.stripChart.configure(xscrollincrement = 1)

        #-------------------------------------------------------------------------
        # Set up Y-axis in first canvas:
        yAxis.create_line(50, self.scaleYval(Y_MIN),
                          50, self.scaleYval(Y_MAX), width = 3)
        # Draw ticks:
        for t in GRID_DARK_L:
            yAxis.create_line(45, self.scaleYval(t),
                              60, self.scaleYval(t), width = 3)
            yAxis.create_text(40, self.scaleYval(t), text = '%5.1f' % t, anchor = tk.E)
        for t in GRID_GRAY_L:
            yAxis.create_line(45, self.scaleYval(t),
                              60, self.scaleYval(t), width = 3, fill = "GRAY")        
            yAxis.create_text(40, self.scaleYval(t), text = '%5.1f' % t, anchor = tk.E)
            
    #-------------------------------------------------------------------------
    # Function to insert new point:
    def update(self, newPoint):
        self.stripChart.move(tk.ALL, 1, 0)
        self.stripChart.create_line(0, newPoint, 1, self.prevPoint, width = 2)
        self.prevPoint = newPoint
        self.stripChart.configure(scrollregion = self.stripChart.bbox(tk.ALL))
        
    #-------------------------------------------------------------------------
    # Function to convert a real y value to its corresponding vertical offset
    # on the canvas:            
    def scaleYval(self, nextY):
        return CANVAS_H - int(CANVAS_H*(nextY - Y_MIN)/(Y_MAX - Y_MIN))
    
    #=============================================================================
    # CALLBACK functions:
    #-------------------------------------------------------------------------
    # Callback function needed for start button:
    def startBtnFunc(self):
        self.stop = False
        self.prevPoint = self.scaleYval(F(0.0))
        for ix in range(1,EVALUATIONS_MAX):
            if self.stop == True: break
            #sleep(0.0001)
            self.update(self.scaleYval(F(ix)))
            self.stripChart.update()
            if ix%10 == 1:
                for g in GRID_DARK_L:
                    self.stripChart.create_line(1, self.scaleYval(g),
                                               -9, self.scaleYval(g), width = 2)
                for g in GRID_GRAY_L:
                    self.stripChart.create_line(1, self.scaleYval(g),
                                               -9, self.scaleYval(g),
                                               width = 1, fill = "GRAY")

    #-------------------------------------------------------------------------
    # Callback function needed for stop button:
    def stopBtnFunc(self):
        self.stop = True
        
 #===============================================================================
# Mainline for testing:        
sp = scrollingPlot()
