import numpy as np
from StructBio.Plots.surfacePlotBundle import SurfacePlotBundle
from StructBio.Plots.surfacePlotKit import (TickLists, SurfacePlot, Axes, QuadrantPlanes,
                                            GridLines, TickLabels, plotMultipleSurfaces)
#===============================================================================
# To define a new surface plot bundle do the following:
#   Define m and n defining the resolution of the surface (the higher, the smoother
#                   at the expense of more execution time and a larger pickle file).
#   Define uMin, uMax, vMin, and vMax defining the rectangular domain of the
#                   indpendent variables u and v.
#   Define the surface name (should be a character string that is reasonably short
#                   in case the string is used within a Label widget of a GUI).
#   Define the default color as an RGB 3-tuple with each floating point entry in the range [0.0, 1.0].
#   Define the transparency as a floating point number in the range [0.0, 1.0].
#   Define the functions x(u,v), y(u,v) and z(u,v).  See the nested loops in the script to follow.
#   Notes:
#       An application using the SurfacePlotBundle class can redefine the surfaceName attribute of an object.
#       You may get z as a function of u and v by defining z in terms of x and y.

from chimera import runCommand
runCommand("set bg_color white")

m = 128
n = 64
uMin = -np.pi
uMax = np.pi
vMin = -np.pi/2.0
vMax = np.pi/2

surfaceName = "Ellipsoid"

transparency = 1.0

a = 0.5
b = 0.5
c = 1.5

uVals_L = np.linspace(uMin, uMax, num = m, endpoint = True)
vVals_L = np.linspace(vMin, vMax, num = n, endpoint = True)

xVals_A = np.zeros((m, n), float)
yVals_A = np.zeros((m, n), float)
zVals_A = np.zeros((m, n), float)
colors_A = np.zeros((m, n, 3), float)
transparency_A = np.zeros((m, n), float)



# Ellipsoid:
for i in range(m):
    for j in range(n):
        xVals_A[i, j] = a * np.cos(vVals_L[j]) * np.cos(uVals_L[i])
        yVals_A[i, j] = b * np.cos(vVals_L[j]) * np.sin(uVals_L[i])
        zVals_A[i, j] = c * np.sin(vVals_L[j])
        # Throw in some periodic coloration just for the sake of interest
        colors_A[i, j, :] = np.array(((1.0 + np.cos(3.*vVals_L[j]))/2.0, np.sin(2.* uVals_L[i]) ** 2, (1.0 + np.sin(vVals_L[j]))/2.0))
        transparency_A[i, j] = transparency


spb = SurfacePlotBundle(xVals_A, yVals_A, zVals_A, colors_A, transparency_A, surfaceName = surfaceName)

ticks = TickLists([spb], 10)
surfaceModels_L = plotMultipleSurfaces([spb],ticks, spbColoration = True)



axes = Axes(ticks)
axes.display()

qPs = QuadrantPlanes(ticks)
qPs.display()

gLs = GridLines(ticks)
gLs.display()

tLabels = TickLabels(ticks)
tLabels.setLabels("axesLabels", "black")
tLabels.setLabels("xT_yMax", "red", stride = 1)
tLabels.setLabels("yT_xMax", "forest green", stride = 1)
tLabels.setLabels("zT_xMax", "blue", stride = 1)

response = raw_input("To save surface as pickled data, type Y followed by Enter.\n")
if response == "Y":
    # Launch file dialog:
    import tkFileDialog, pickle
    fileName = tkFileDialog.asksaveasfilename(filetypes =  [('Pickle output', '*.pkl')],
                                              initialfile = surfaceName,
                                              defaultextension = 'pkl',
                                              title = "Save the surface as pickled data. (Click Cancel to avoid save).")

    # Save the SurfacePlotBundle object as pickled data:
    if len(fileName) > 0:
        fileHandler = open(fileName, "wb")
        pickle.dump(spb, fileHandler, -1)
        fileHandler.close()




